﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Reflection;

namespace PDSA_Socket
{
    //
    //  PDS-A_Socket C# Sample : Ver 1.1.1
    //  Copyright(c) 2012 TEXIO TECHNOLOGY CORPORATION
    //

    /// <summary>メイン画面クラス
    /// </summary>
    public partial class PDSA_Socket : Form
    {
        private System.Net.Sockets.TcpClient m_TcpClient;           // TCPクライアント
        private System.Net.Sockets.NetworkStream m_NetworkStream;   // ネットワークストリーム
        private string m_IpFileName;								// 登録済みのIPアドレスの保存

        /// <summary>コンストラクタ
        /// </summary>
        public PDSA_Socket()
        {
            InitializeComponent();

            m_IpFileName = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location);
            m_IpFileName += "\\PDSA_Socket.txt";				// 環境を保存するファイル名
            IpSaveFileRead();								    // m_IpFileNameのファイル内のアドレス情報の読み込み
            PDSA_CommButtonEnable(0);
        }

        private void PDSA_Socket_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (this.PictureOnBox.Visible == true)
            {
                SendDataSet(false, "GTL");                       // リモート解除
                PDSA_CommButtonEnable(2);                        // 切断待ち
                this.LabelReciveWork.Text = " Close";
                this.Refresh();
                timerSocket.Enabled = false;
                System.Threading.Thread.Sleep(500);
                SocketClose();
            }
        }

        /// <summary>ボタンのON/OFF表示処理
        /// </summary>
        private void PDSA_CommButtonEnable(int mode)
        {
            Boolean sel;

            sel = mode == 1 || mode == 3 ? true : false;
            this.groupBoxSend.Enabled = sel;

            sel = mode == 2 || mode == 3 ? true : false;
            this.PictureOnOfBox.Visible = sel;

            sel= mode == 1 ? true : false;
            this.PictureOnBox.Visible = sel;
            this.groupBoxOutput.Enabled = sel;
            this.groupBoxPreset.Enabled = sel;
            this.groupBoxMonitor.Enabled = sel;
            this.CommandInformationBt.Enabled = sel;
            this.CommandRatedBt.Enabled = sel;
            this.CommandRemOffBt.Enabled = sel;
            this.CommandDevCloseBt.Enabled = sel;

            sel = mode == 0 ? true : false;
            this.TextIpAddr0Bx.Enabled = sel;
            this.TextIpAddr1Bx.Enabled = sel;
            this.TextIpAddr2Bx.Enabled = sel;
            this.TextIpAddr3Bx.Enabled = sel;
            this.NumericPortNoUpDown.Enabled = sel;
            this.CommandDevOpenBt.Enabled = sel;
            this.PictureOffBox.Visible = sel;
        }
 
        //***********************************************************************
        //  Processing : Open / Close Button
        //***********************************************************************

        private void CommandDevOpenBt_Click(object sender, EventArgs e)
        {
            PDSA_CommButtonEnable(2);                       // 接続待ち
            LabelReciveWork.Text = " Open";
            this.Refresh();
            try
            {
                if (SocketOpen() == false)
                {
                    LabelReciveWork.Text = " Connect Error";
                    PDSA_CommButtonEnable(0);                // 切断状態
                    return;
                }
                System.Threading.Thread.Sleep(500);
                LabelReciveWork.Text = "";
                SendDataSet(false, "ADRS 0");                // ローカルアドレス指定
                SendDataSet(false, "MONDSP 0");              // 通常表示
                PDSA_CommButtonEnable(1);                    // 接続状態
                this.Refresh();
                System.Threading.Thread.Sleep(100);
                timerSocket.Enabled = true;
                TextSendBx.Text = "";
            }
            catch (Exception ex)
            {
                LabelReciveWork.Text = " Connect Error";
                timerSocket.Enabled = false;
                SocketClose();
                PDSA_CommButtonEnable(0);                    // 切断状態
            }
        }

        private void CommandDevCloseBt_Click(object sender, EventArgs e)
        {
            SendDataSet(false, "GTL");                      // リモート解除
            PDSA_CommButtonEnable(2);                       // 切断待ち
            LabelReciveWork.Text = " Close";
            this.Refresh();
            timerSocket.Enabled = false;
            System.Threading.Thread.Sleep(500);
            SocketClose();
            TextSendBx.Text = "";
            LabelReciveWork.Text = "";
            PDSA_CommButtonEnable(0);                       // 切断状態
        }

        //***********************************************************************
        //  Processing : Command Button
        //***********************************************************************
 
        private void CommandSaveBt_Click(object sender, EventArgs e)
        {
            IpSaveFileWrite();
        }

        private void CommandOutOnBt_Click(object sender, EventArgs e)
        {
            SendDataSet(true, "OUTPUT 1");
        }

        private void CommandOutOffBt_Click(object sender, EventArgs e)
        {
            SendDataSet(true, "OUTPUT 0");
        }

        private void CommandPreset1Bt_Click(object sender, EventArgs e)
        {
            SendDataSet(true, "PRESET 1");
        }

        private void CommandPreset2Bt_Click(object sender, EventArgs e)
        {
            SendDataSet(true, "PRESET 2");
        }

        private void CommandPreset3Bt_Click(object sender, EventArgs e)
        {
            SendDataSet(true, "PRESET 3");
        }

        private void CommandMoniVoltBt_Click(object sender, EventArgs e)
        {
            SendDataSet(true, "VREAD?");
        }

        private void CommandMoniCurrBt_Click(object sender, EventArgs e)
        {
            SendDataSet(true, "AREAD?");
        }

        private void CommandInformationBt_Click(object sender, EventArgs e)
        {
            SendDataSet(true, "*IDN?");
        }

        private void CommandRatedBt_Click(object sender, EventArgs e)
        {
            SendDataSet(true, "MODEL?");
        }

        private void CommandRemOffBt_Click(object sender, EventArgs e)
        {
            SendDataSet(true, "GTL");
        }

        private void CommandSendBt_Click(object sender, EventArgs e)
        {
            SendDataSet(false, TextSendBx.Text);
        }

        private void CommandClearBt_Click(object sender, EventArgs e)
        {
            ListReceiveBx.Items.Clear();
        }

 
        private void  SendDataSet(Boolean sel, String buf)
        {
            if (sel == true)
            {
                TextSendBx.Text = buf;
            }
            if (0 < buf.Length)
            {
                SocketSendData(buf + "\n");
            }
        }

        //***********************************************************************
        //  Processing : IP Address TextBox
        //***********************************************************************

        private void TextIpAddrBx_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (!('0' <= e.KeyChar && e.KeyChar <= '9'))
            {
                if (e.KeyChar != 8)
                {
                    e.KeyChar = '\0';

                }
            }
        }

        private void TextIpAddrBx_TextChanged(object sender, EventArgs e)
        {
            TextBox dat;

            dat = (TextBox)sender;
            if (3 < dat.Text.Length)
            {
                int pos = dat.SelectionStart;
                dat.Text = dat.Text.Substring(1);
                if (1 < pos)
                {
                    dat.SelectionStart = pos - 1;
                }
            }
        }
        
        private void TextIpAddrBx_Leave(object sender, EventArgs e)
        {
            TextBox dat;

            dat = (TextBox)sender;
            if (0 == dat.Text.Length)
            {
                dat.Text = "0";
            }
            else if (255 < int.Parse(dat.Text))
            {
                dat.Text = "255";
            }
        }


        //***********************************************************************
        //  Processing : File
        //***********************************************************************
        /// <summary>前回保存した環境を読み込む
        /// </summary>
        private void IpSaveFileRead()
        {
            try
            {
                if (System.IO.File.Exists(m_IpFileName))
                {             // ファイルが存在しているかどうか確認する
                    string buf0, buf1, buf2, buf3, buf4;
                    System.IO.StreamReader cReader = new System.IO.StreamReader(
                                    m_IpFileName, System.Text.Encoding.GetEncoding("shift_jis"));
                    buf0 = cReader.ReadLine();
                    buf1 = cReader.ReadLine();
                    buf2 = cReader.ReadLine();
                    buf3 = cReader.ReadLine();
                    buf4 = cReader.ReadLine();
                    cReader.Close();                            // cReaderを閉じてオブジェクトの破棄する
                    TextIpAddr0Bx.Text = buf0;
                    TextIpAddr1Bx.Text = buf1;
                    TextIpAddr2Bx.Text = buf2;
                    TextIpAddr3Bx.Text = buf3;
                    NumericPortNoUpDown.Value = int.Parse(buf4);
                }
            }
            catch (Exception ex)
            {
                TextIpAddr0Bx.Text = "192";
                TextIpAddr1Bx.Text = "168";
                TextIpAddr2Bx.Text = "1";
                TextIpAddr3Bx.Text = "1";
                NumericPortNoUpDown.Value = 10001;
            }
        }

        /// <summary>現在の設定環境を保存する
        /// </summary>
        private void IpSaveFileWrite()
        {
            try
            {
                System.IO.StreamWriter cWriter = new System.IO.StreamWriter(
                    m_IpFileName, false, System.Text.Encoding.GetEncoding("shift_jis"));   // ファイルを上書き保存で開く
                cWriter.Write(TextIpAddr0Bx.Text + System.Environment.NewLine);
                cWriter.Write(TextIpAddr1Bx.Text + System.Environment.NewLine);
                cWriter.Write(TextIpAddr2Bx.Text + System.Environment.NewLine);
                cWriter.Write(TextIpAddr3Bx.Text + System.Environment.NewLine);
                cWriter.Write(NumericPortNoUpDown.Value.ToString() + System.Environment.NewLine);
                cWriter.Close();        // cWriterを閉じてオブジェクトの破棄する
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, MethodBase.GetCurrentMethod().ReflectedType.FullName + "." + MethodBase.GetCurrentMethod().Name, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }


        //***********************************************************************/
        // VB.NETのソケット通信サンプル
        //   Public Class Form1
        // ソケット生成
        //***********************************************************************/

        // ソケット接続
        private Boolean SocketOpen()
        {
            this.m_NetworkStream = null;
            this.m_TcpClient = null;

            try
            {
                // インスタンス生成
                this.m_TcpClient = new System.Net.Sockets.TcpClient();

                PDSA_CommButtonEnable(2);
                this.groupBoxConnect.Refresh();

                // 文字列の先頭0削除
                string strIpAddress = TextIpAddr0Bx.Text + "." +
                                      TextIpAddr1Bx.Text + "." +
                                      TextIpAddr2Bx.Text + "." +
                                      TextIpAddr3Bx.Text;

                // TCPクライアント接続
                this.m_TcpClient.Connect(strIpAddress,                                      // IPアドレス
                                         Convert.ToInt32(this.NumericPortNoUpDown.Value));  // ポート番号

                // ソケットストリーム取得
                this.m_NetworkStream = this.m_TcpClient.GetStream();
                return (true);
           }
            catch (Exception ex)
            {
                PDSA_CommButtonEnable(0);
                this.m_NetworkStream = null;
                this.m_TcpClient = null;
                return (false);
            }

        }

        // ソケット切断
        private void SocketClose()
        {
            // オブジェクト存在（真）&& TCPクライアント接続（真）
            if (this.m_TcpClient != null && this.m_TcpClient.Connected == true)
            {
                // ソケットストリームCLOSE
                this.m_NetworkStream.Close();

                // TCPクライアントCLOSE
                this.m_TcpClient.Close();

            }
              // オブジェクト開放
            this.m_NetworkStream = null;
            this.m_TcpClient = null;
        }

        // ソケット送信
        private void SocketSendData(string buf)
        {
            if (0 < buf.Length)
            {
                    // 送信データバッファ
                Byte[] DataBuff = System.Text.Encoding.GetEncoding("SHIFT-JIS").GetBytes(buf);

                    // データ送信
                    this.m_NetworkStream.Write(DataBuff,                // 送信データバッファ
                                               0,                       // オフセット
                                               DataBuff.GetLength(0));  // 送信データ数
             }
        }

        // ソケット受信
        private void SocketReadData()
        {
            // TCP受信データ数取得
            int DataCnt = this.m_TcpClient.Available;

            // TCP受信データ存在（真）
            if (DataCnt > 0)
            {
                // 受信データバッファ
                int ln;
                Byte[] DataBuff = new Byte[DataCnt];
                Byte[] Data1Buff = new Byte[1];			    // ASC文字変換用バッファ
                // TCPデータ受信
                this.m_NetworkStream.Read(DataBuff,         // 受信データバッファ
                                          0,                // オフセット
                                          DataCnt);         // 受信データ数

                // Shift JISとして文字列に変換
                for (ln = 0; ln < DataCnt; ln++)
                {
                    if ('\n' == DataBuff[ln])
                    {			                            // 改行コードの終了コードの確認
                        if (this.LabelReciveWork.Text != "")
                        {							        // 受信バッファのデータの有無
                            this.ListReceiveBx.Items.Add(this.LabelReciveWork.Text);	// 受信バッファのデータをリストにコピー
                            this.LabelReciveWork.Text = "";								// 受信バッファの初期化
                        }
                    }
                    else
                    {
                        Data1Buff[0] = DataBuff[ln];									// 受信バッファに文字を設定する
                        this.LabelReciveWork.Text += System.Text.Encoding.GetEncoding(932).GetString(Data1Buff);
                    }
                }
                if (10 < this.ListReceiveBx.Items.Count)
                {	// 最大表示可能項目数の確認
                    this.ListReceiveBx.Items.RemoveAt(0);	// 先頭の項目を削除する
                }
            }
        }

        private void timerSocket_Tick(object sender, EventArgs e)
        {
            SocketReadData();
        }

    }
}
